@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,11 +21,15 @@
 /*!
   @file    jpgimage.hpp
   @brief   Class JpegImage to access JPEG images
-  @version $Rev: 1750 $
+  @version $Rev: 2045 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @author  Brad Schick (brad)
            <a href="mailto:brad@robotbattle.com">brad@robotbattle.com</a>
+  @author  Volker Grabsch (vog)
+           <a href="mailto:vog@notjusthosting.com">vog@notjusthosting.com</a>
+  @author  Michael Ulbrich (mul)
+           <a href="mailto:mul@rentapacs.de">mul@rentapacs.de</a>
   @date    15-Jan-05, brad: split out from image.cpp
  */
 #ifndef JPGIMAGE_HPP_
@@ -63,6 +67,16 @@ namespace Exiv2 {
         static const char     bimId_[]; //!< %Photoshop marker
         static const uint16_t iptc_;    //!< %Photoshop IPTC marker
 
+        /*!
+          @brief Validates all IRBs
+
+          @param pPsData        Existing IRB buffer
+          @param sizePsData     Size of the IRB buffer, may be 0
+          @return true  if all IRBs are valid;<BR>
+                  false otherwise
+        */
+        static bool valid(const byte* pPsData,
+                          long        sizePsData);
         /*!
           @brief Locates the data for a %Photoshop tag in a %Photoshop formated memory
               buffer. Operates on raw data to simplify reuse.
@@ -280,7 +294,7 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        std::string mimeType() const { return "image/jpeg"; }
+        std::string mimeType() const;
         //@}
 
     protected:
@@ -341,7 +355,7 @@ namespace Exiv2 {
         //@}
         //! @name Accessors
         //@{
-        std::string mimeType() const { return "image/x-exv"; }
+        std::string mimeType() const;
         //@}
 
     protected:
