@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2006-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -22,7 +22,7 @@
   @file    mrwimage.hpp
   @brief   Minolta RAW image, implemented using the following references:
            <a href="http://www.dalibor.cz/minolta/raw_file_format.htm">Minolta Raw file format</a> by Dalibor Jelinek
-  @version $Rev: 1064 $
+  @version $Rev: 2045 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    13-May-06, ahu: created
@@ -34,6 +34,7 @@
 // included header files
 #include "image.hpp"
 #include "basicio.hpp"
+#include "types.hpp"
 
 // + standard includes
 #include <string>
@@ -54,7 +55,7 @@ namespace Exiv2 {
       @brief Class to access raw Minolta MRW images. Exif metadata is supported
              directly, IPTC is read from the Exif data, if present.
      */
-    class MrwImage : public Image {
+    class EXIV2API MrwImage : public Image {
     public:
         //! @name Creators
         //@{
@@ -85,12 +86,12 @@ namespace Exiv2 {
          */
         void writeMetadata();
         /*!
-          @brief Todo: Not supported yet, requires writeMetadata(). Calling 
+          @brief Todo: Not supported yet, requires writeMetadata(). Calling
               this function will throw an Error(32).
          */
         void setExifData(const ExifData& exifData);
         /*!
-          @brief Todo: Not supported yet, requires writeMetadata(). Calling 
+          @brief Todo: Not supported yet, requires writeMetadata(). Calling
               this function will throw an Error(32).
          */
         void setIptcData(const IptcData& iptcData);
@@ -103,7 +104,9 @@ namespace Exiv2 {
 
         //! @name Accessors
         //@{
-        std::string mimeType() const { return "image/x-minolta-mrw"; }
+        std::string mimeType() const;
+        int pixelWidth() const;
+        int pixelHeight() const;
         //@}
 
     private:
@@ -127,10 +130,10 @@ namespace Exiv2 {
              Caller owns the returned object and the auto-pointer ensures that
              it will be deleted.
      */
-    Image::AutoPtr newMrwInstance(BasicIo::AutoPtr io, bool create);
+    EXIV2API Image::AutoPtr newMrwInstance(BasicIo::AutoPtr io, bool create);
 
     //! Check if the file iIo is a MRW image.
-    bool isMrwType(BasicIo& iIo, bool advance);
+    EXIV2API bool isMrwType(BasicIo& iIo, bool advance);
 
 }                                       // namespace Exiv2
 
