@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    tags.hpp
   @brief   Exif tag and type information
-  @version $Rev: 1118 $
+  @version $Rev: 2052 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    15-Jan-04, ahu: created<BR>
@@ -48,15 +48,17 @@ namespace Exiv2 {
 
 // *****************************************************************************
 // class declarations
+    class ExifData;
     class Value;
-    class Entry;
+    struct TagInfo;
 
 // *****************************************************************************
 // type definitions
 
     //! Type for a function pointer for functions interpreting the tag value
-    typedef std::ostream& (*PrintFct)(std::ostream&, const Value&);
-
+    typedef std::ostream& (*PrintFct)(std::ostream&, const Value&, const ExifData* pExifData);
+    //! A function returning a tag list.
+    typedef const TagInfo* (*TagListFct)();
     /*!
       @brief Section identifiers to logically group tags. A section consists
              of nothing more than a name, based on the Exif standard.
@@ -64,33 +66,39 @@ namespace Exiv2 {
     enum SectionId { sectionIdNotSet,
                      imgStruct, recOffset, imgCharacter, otherTags, exifFormat,
                      exifVersion, imgConfig, userInfo, relatedFile, dateTime,
-                     captureCond, gpsTags, iopTags, makerTags,
+                     captureCond, gpsTags, iopTags, makerTags, dngTags, panaRaw,
+                     tiffEp, tiffPm6, adobeOpi,
                      lastSectionId };
 
 // *****************************************************************************
 // class definitions
 
-    //! Contains information pertaining to one IFD
-    struct IfdInfo {
-        //! Constructor
-        IfdInfo(IfdId ifdId, const char* name, const char* item);
+    //! The details of an IFD.
+    struct EXIV2API IfdInfo {
+        struct Item;
+        bool operator==(IfdId ifdId) const;     //!< Comparison operator for IFD id
+        bool operator==(const Item& item) const;       //!< Comparison operator for IFD item
         IfdId ifdId_;                           //!< IFD id
         const char* name_;                      //!< IFD name
-        //! Related IFD item. This is also an IFD name, unique for each IFD.
-        const char* item_;
+        const char* item_; //!< Related IFD item. This is also an IFD name, unique for each IFD.
+        TagListFct tagList_;                    //!< Tag list
     };
 
-    //! Contains information pertaining to one section
-    struct SectionInfo {
-        //! Constructor
-        SectionInfo(SectionId sectionId, const char* name, const char* desc);
+    //! Search key to find an IfdInfo by its IFD item.
+    struct EXIV2API IfdInfo::Item {
+        Item(const std::string& item);          //!< Constructor
+        std::string i_;                         //!< IFD item
+    };
+
+    //! The details of a section.
+    struct EXIV2API SectionInfo {
         SectionId sectionId_;                   //!< Section id
         const char* name_;                      //!< Section name (one word)
         const char* desc_;                      //!< Section description
     };
 
     //! Tag information
-    struct TagInfo {
+    struct EXIV2API TagInfo {
         //! Constructor
         TagInfo(
             uint16_t tag,
@@ -116,7 +124,7 @@ namespace Exiv2 {
       @brief Helper structure for lookup tables for translations of numeric
              tag values to human readable labels.
      */
-    struct TagDetails {
+    struct EXIV2API TagDetails {
         long val_;                              //!< Tag value
         const char* label_;                     //!< Translation of the tag value
 
@@ -128,17 +136,35 @@ namespace Exiv2 {
       @brief Helper structure for lookup tables for translations of bitmask
              values to human readable labels.
      */
-    struct TagDetailsBitmask {
+    struct EXIV2API TagDetailsBitmask {
         uint32_t mask_;                         //!< Bitmask value
         const char* label_;                     //!< Description of the tag value
     }; // struct TagDetailsBitmask
 
     /*!
-      @brief Generic print function to translate a long value to a description
+      @brief Helper structure for lookup tables for translations of controlled
+             vocabulary strings to their descriptions.
+     */
+    struct EXIV2API TagVocabulary {
+        const char* voc_;                       //!< Vocabulary string
+        const char* label_;                     //!< Description of the vocabulary string
+
+        /*!
+          @brief Comparison operator for use with the find template
+
+          Compare vocabulary strings like "PR-NON" with keys like
+          "http://ns.useplus.org/ldf/vocab/PR-NON" and return true if the vocabulary
+          string matches the end of the key.
+         */
+        bool operator==(const std::string& key) const;
+    }; // struct TagDetails
+
+    /*!
+      @brief Generic pretty-print function to translate a long value to a description
              by looking up a reference table.
      */
     template <int N, const TagDetails (&array)[N]>
-    std::ostream& printTag(std::ostream& os, const Value& value)
+    std::ostream& printTag(std::ostream& os, const Value& value, const ExifData*)
     {
         const TagDetails* td = find(array, value.toLong());
         if (td) {
@@ -158,11 +184,15 @@ namespace Exiv2 {
              by looking up bitmasks in a reference table.
      */
     template <int N, const TagDetailsBitmask (&array)[N]>
-    std::ostream& printTagBitmask(std::ostream& os, const Value& value)
+    std::ostream& printTagBitmask(std::ostream& os, const Value& value, const ExifData*)
     {
         const uint32_t val = static_cast<uint32_t>(value.toLong());
+        if (val == 0 && N > 0) {
+            const TagDetailsBitmask* td = *(&array);
+            if (td->mask_ == 0) return os << exvGettext(td->label_);
+        }
         bool sep = false;
-        for (int i = 0; i < N; i++) {
+        for (int i = 0; i < N; ++i) {
             // *& acrobatics is a workaround for a MSVC 7.1 bug
             const TagDetailsBitmask* td = *(&array) + i;
 
@@ -182,8 +212,28 @@ namespace Exiv2 {
 //! Shortcut for the printTagBitmask template which requires typing the array name only once.
 #define EXV_PRINT_TAG_BITMASK(array) printTagBitmask<EXV_COUNTOF(array), array>
 
-    //! Container for Exif tag information. Implemented as a static class.
-    class ExifTags {
+    /*!
+      @brief Generic pretty-print function to translate a controlled vocabulary value (string)
+             to a description by looking up a reference table.
+     */
+    template <int N, const TagVocabulary (&array)[N]>
+    std::ostream& printTagVocabulary(std::ostream& os, const Value& value, const ExifData*)
+    {
+        const TagVocabulary* td = find(array, value.toString());
+        if (td) {
+            os << exvGettext(td->label_);
+        }
+        else {
+            os << "(" << value << ")";
+        }
+        return os;
+    }
+
+//! Shortcut for the printTagVocabulary template which requires typing the array name only once.
+#define EXV_PRINT_VOCABULARY(array) printTagVocabulary<EXV_COUNTOF(array), array>
+
+    //! Exif tag reference, implemented as a static class.
+    class EXIV2API ExifTags {
         //! Prevent construction: not implemented.
         ExifTags();
         //! Prevent copy-construction: not implemented.
@@ -271,7 +321,8 @@ namespace Exiv2 {
         static std::ostream& printTag(std::ostream& os,
                                       uint16_t tag,
                                       IfdId ifdId,
-                                      const Value& value);
+                                      const Value& value,
+                                      const ExifData* pExifData =0);
         //! Return read-only list of built-in IFD0/1 tags
         static const TagInfo* ifdTagList();
         //! Return read-only list of built-in Exif IFD tags
@@ -280,46 +331,40 @@ namespace Exiv2 {
         static const TagInfo* iopTagList();
         //! Return read-only list of built-in GPS tags
         static const TagInfo* gpsTagList();
+        //! Return read-only list of built-in Exiv2 Makernote info tags
+        static const TagInfo* mnTagList();
         //! Print a list of all standard Exif tags to output stream
         static void taglist(std::ostream& os);
-        //! Print a list of all tags related to one makernote %IfdId
-        static void makerTaglist(std::ostream& os, IfdId ifdId);
-        //! Register an %IfdId with the base IFD %TagInfo list for a makernote
-        static void registerBaseTagInfo(IfdId ifdId);
+        //! Print the list of tags for \em %IfdId
+        static void taglist(std::ostream& os, IfdId ifdId);
         /*!
-          @brief Register an %IfdId and %TagInfo list for a makernote
-
-          @throw Error if the MakerTagInfo registry is full
-         */
-        static void registerMakerTagInfo(IfdId ifdId, const TagInfo* tagInfo);
-        /*!
-          @brief Return true if \em ifdId is an %Ifd Id which is registered
-                 as a makernote %Ifd id. Note: Calling this function with
+          @brief Return true if \em ifdId is an %Ifd id which is
+                 a makernote %Ifd id. Note: Calling this function with
                  makerIfd returns false.
         */
         static bool isMakerIfd(IfdId ifdId);
+        /*!
+          @brief Return true if \em ifdId is an Exif %Ifd Id, i.e., one of
+                 ifd0Id, exifIfdId, gpsIfdId, iopIfdId or ifd1Id, else false.
+                 This is used to differentiate between standard Exif %Ifds
+                 and %Ifds associated with the makernote.
+        */
+        static bool isExifIfd(IfdId ifdId);
 
     private:
-        static int tagInfoIdx(uint16_t tag, IfdId ifdId);
-        static const TagInfo* makerTagInfo(uint16_t tag, IfdId ifdId);
-        static const TagInfo* makerTagInfo(const std::string& tagName,
-                                           IfdId ifdId);
+        static const TagInfo* tagList(IfdId ifdId);
+        static const TagInfo* tagInfo(uint16_t tag, IfdId ifdId);
+        static const TagInfo* tagInfo(const std::string& tagName, IfdId ifdId);
 
-        static const IfdInfo     ifdInfo_[];
-        static const SectionInfo sectionInfo_[];
-
-        static const TagInfo*    tagInfos_[];
-
-        static const int         MAX_MAKER_TAG_INFOS = 64;
-        static const TagInfo*    makerTagInfos_[MAX_MAKER_TAG_INFOS];
-        static IfdId             makerIfdIds_[MAX_MAKER_TAG_INFOS];
+        static const IfdInfo     ifdInfo_[];     //!< All Exif and Makernote tag lists
+        static const SectionInfo sectionInfo_[]; //!< Exif (and one common Makernote) sections
 
     }; // class ExifTags
 
     /*!
       @brief Concrete keys for Exif metadata.
      */
-    class ExifKey : public Key {
+    class EXIV2API ExifKey : public Key {
     public:
         //! Shortcut for an %ExifKey auto pointer.
         typedef std::auto_ptr<ExifKey> AutoPtr;
@@ -345,8 +390,6 @@ namespace Exiv2 {
                  item parameters.
          */
         ExifKey(uint16_t tag, const std::string& ifdItem);
-        //! Constructor to build an ExifKey from an IFD entry.
-        explicit ExifKey(const Entry& e);
         //! Copy constructor
         ExifKey(const ExifKey& rhs);
         virtual ~ExifKey();
@@ -358,20 +401,22 @@ namespace Exiv2 {
           @brief Assignment operator.
          */
         ExifKey& operator=(const ExifKey& rhs);
+        //! Set the index.
+        void setIdx(int idx);
         //@}
 
         //! @name Accessors
         //@{
-        virtual std::string key() const { return key_; }
-        virtual const char* familyName() const { return familyName_; }
+        virtual std::string key() const;
+        virtual const char* familyName() const;
         /*!
           @brief Return the name of the group (the second part of the key).
                  For Exif keys, the group name is the IFD item.
         */
-        virtual std::string groupName() const { return ifdItem(); }
+        virtual std::string groupName() const;
         virtual std::string tagName() const;
         virtual std::string tagLabel() const;
-        virtual uint16_t tag() const { return tag_; }
+        virtual uint16_t tag() const;
 
         AutoPtr clone() const;
         //! Return the IFD id
@@ -382,7 +427,7 @@ namespace Exiv2 {
         std::string ifdItem() const { return ifdItem_; }
         //! Return the name of the Exif section (deprecated)
         std::string sectionName() const;
-        //! Return the index (unique id of this key within the original IFD)
+        //! Return the index (unique id of this key within the original Exif data, 0 if not set)
         int idx() const { return idx_; }
         //@}
 
@@ -406,7 +451,7 @@ namespace Exiv2 {
 
     private:
         //! Internal virtual copy constructor.
-        virtual ExifKey* clone_() const;
+        EXV_DLLLOCAL virtual ExifKey* clone_() const;
 
         // DATA
         static const char* familyName_;
@@ -414,74 +459,123 @@ namespace Exiv2 {
         uint16_t tag_;                  //!< Tag value
         IfdId ifdId_;                   //!< The IFD associated with this tag
         std::string ifdItem_;           //!< The IFD item
-        int idx_;                       //!< Unique id of an entry within one IFD
+        int idx_;                       //!< Unique id of the Exif key in the image
         std::string key_;               //!< Key
     }; // class ExifKey
 
 // *****************************************************************************
 // free functions
 
-    /*!
-      @brief Return true if \em ifdId is an Exif %Ifd Id, i.e., one of
-             ifd0Id, exifIfdId, gpsIfdId, iopIfdId or ifd1Id, else false.
-             This is used to differentiate between standard Exif %Ifds
-             and %Ifds associated with the makernote.
-     */
-    bool isExifIfd(IfdId ifdId);
-
     //! Output operator for TagInfo
-    std::ostream& operator<<(std::ostream& os, const TagInfo& ti);
+    EXIV2API std::ostream& operator<<(std::ostream& os, const TagInfo& ti);
 
     //! @name Functions printing interpreted tag values
     //@{
     //! Default print function, using the Value output operator
-    std::ostream& printValue(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& printValue(std::ostream& os, const Value& value, const ExifData*);
     //! Print the value converted to a long
-    std::ostream& printLong(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& printLong(std::ostream& os, const Value& value, const ExifData*);
     //! Print a Rational or URational value in floating point format
-    std::ostream& printFloat(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& printFloat(std::ostream& os, const Value& value, const ExifData*);
     //! Print a longitude or latitude value
-    std::ostream& printDegrees(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& printDegrees(std::ostream& os, const Value& value, const ExifData*);
     //! Print function converting from UCS-2LE to UTF-8
-    std::ostream& printUcs2(std::ostream& os, const Value& value);
-
+    EXIV2API std::ostream& printUcs2(std::ostream& os, const Value& value, const ExifData*);
+    //! Print function for Exif units
+    EXIV2API std::ostream& printExifUnit(std::ostream& os, const Value& value, const ExifData*);
+    //! Print GPS version
+    EXIV2API std::ostream& print0x0000(std::ostream& os, const Value& value, const ExifData*);
+    //! Print GPS altitude ref
+    EXIV2API std::ostream& print0x0005(std::ostream& os, const Value& value, const ExifData*);
     //! Print GPS altitude
-    std::ostream& print0x0006(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x0006(std::ostream& os, const Value& value, const ExifData*);
     //! Print GPS timestamp
-    std::ostream& print0x0007(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x0007(std::ostream& os, const Value& value, const ExifData*);
+    //! Print GPS status
+    EXIV2API std::ostream& print0x0009(std::ostream& os, const Value& value, const ExifData*);
+    //! Print GPS measurement mode
+    EXIV2API std::ostream& print0x000a(std::ostream& os, const Value& value, const ExifData*);
+    //! Print GPS speed ref
+    EXIV2API std::ostream& print0x000c(std::ostream& os, const Value& value, const ExifData*);
+    //! Print GPS destination distance ref
+    EXIV2API std::ostream& print0x0019(std::ostream& os, const Value& value, const ExifData*);
+    //! Print GPS differential correction
+    EXIV2API std::ostream& print0x001e(std::ostream& os, const Value& value, const ExifData*);
+    //! Print orientation
+    EXIV2API std::ostream& print0x0112(std::ostream& os, const Value& value, const ExifData*);
+    //! Print YCbCrPositioning
+    EXIV2API std::ostream& print0x0213(std::ostream& os, const Value& value, const ExifData*);
     //! Print the copyright
-    std::ostream& print0x8298(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x8298(std::ostream& os, const Value& value, const ExifData*);
     //! Print the exposure time
-    std::ostream& print0x829a(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x829a(std::ostream& os, const Value& value, const ExifData*);
     //! Print the f-number
-    std::ostream& print0x829d(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x829d(std::ostream& os, const Value& value, const ExifData*);
+    //! Print exposure program
+    EXIV2API std::ostream& print0x8822(std::ostream& os, const Value& value, const ExifData*);
     //! Print ISO speed ratings
-    std::ostream& print0x8827(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x8827(std::ostream& os, const Value& value, const ExifData*);
     //! Print components configuration specific to compressed data
-    std::ostream& print0x9101(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x9101(std::ostream& os, const Value& value, const ExifData*);
     //! Print exposure time converted from APEX shutter speed value
-    std::ostream& print0x9201(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x9201(std::ostream& os, const Value& value, const ExifData*);
     //! Print f-number converted from APEX aperture value
-    std::ostream& print0x9202(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x9202(std::ostream& os, const Value& value, const ExifData*);
     //! Print the exposure bias value
-    std::ostream& print0x9204(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x9204(std::ostream& os, const Value& value, const ExifData*);
     //! Print the subject distance
-    std::ostream& print0x9206(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x9206(std::ostream& os, const Value& value, const ExifData*);
+    //! Print metering mode
+    EXIV2API std::ostream& print0x9207(std::ostream& os, const Value& value, const ExifData*);
+    //! Print light source
+    EXIV2API std::ostream& print0x9208(std::ostream& os, const Value& value, const ExifData*);
     //! Print the actual focal length of the lens
-    std::ostream& print0x920a(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x920a(std::ostream& os, const Value& value, const ExifData*);
     //! Print the user comment
-    std::ostream& print0x9286(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0x9286(std::ostream& os, const Value& value, const ExifData*);
+    //! Print color space
+    EXIV2API std::ostream& print0xa001(std::ostream& os, const Value& value, const ExifData*);
+    //! Print sensing method
+    EXIV2API std::ostream& print0xa217(std::ostream& os, const Value& value, const ExifData*);
+    //! Print file source
+    EXIV2API std::ostream& print0xa300(std::ostream& os, const Value& value, const ExifData*);
+    //! Print scene type
+    EXIV2API std::ostream& print0xa301(std::ostream& os, const Value& value, const ExifData*);
+    //! Print custom rendered
+    EXIV2API std::ostream& print0xa401(std::ostream& os, const Value& value, const ExifData*);
+    //! Print exposure mode
+    EXIV2API std::ostream& print0xa402(std::ostream& os, const Value& value, const ExifData*);
+    //! Print white balance
+    EXIV2API std::ostream& print0xa403(std::ostream& os, const Value& value, const ExifData*);
     //! Print digital zoom ratio
-    std::ostream& print0xa404(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0xa404(std::ostream& os, const Value& value, const ExifData*);
     //! Print 35mm equivalent focal length
-    std::ostream& print0xa405(std::ostream& os, const Value& value);
+    EXIV2API std::ostream& print0xa405(std::ostream& os, const Value& value, const ExifData*);
+    //! Print scene capture type
+    EXIV2API std::ostream& print0xa406(std::ostream& os, const Value& value, const ExifData*);
+    //! Print gain control
+    EXIV2API std::ostream& print0xa407(std::ostream& os, const Value& value, const ExifData*);
+    //! Print saturation
+    EXIV2API std::ostream& print0xa409(std::ostream& os, const Value& value, const ExifData*);
+    //! Print subject distance range
+    EXIV2API std::ostream& print0xa40c(std::ostream& os, const Value& value, const ExifData*);
+    //! Print GPS direction ref
+    EXIV2API std::ostream& printGPSDirRef(std::ostream& os, const Value& value, const ExifData*);
+    //! Print contrast, sharpness (normal, soft, hard)
+    EXIV2API std::ostream& printNormalSoftHard(std::ostream& os, const Value& value, const ExifData*);
+    //! Print any version packed in 4 Bytes format : major major minor minor
+    EXIV2API std::ostream& printExifVersion(std::ostream& os, const Value& value, const ExifData*);
+    //! Print any version encoded in the ASCII string majormajorminorminor
+    EXIV2API std::ostream& printXmpVersion(std::ostream& os, const Value& value, const ExifData*);
+    //! Print a date following the format YYYY-MM-DDTHH:MM:SSZ
+    EXIV2API std::ostream& printXmpDate(std::ostream& os, const Value& value, const ExifData*);
     //@}
 
     //! Calculate F number from an APEX aperture value
-    float fnumber(float apertureValue);
+    EXIV2API float fnumber(float apertureValue);
 
     //! Calculate the exposure time from an APEX shutter speed value
-    URational exposureTime(float shutterSpeedValue);
+    EXIV2API URational exposureTime(float shutterSpeedValue);
 
 }                                       // namespace Exiv2
 
